# Submits input asynchronously to agents, then loops through their responses 
# until it finds a high scoring response, or until a time limit is reached. 

# The alternate require paths are relative to the redturnip root directory.
begin require 'gagent'; rescue LoadError; require 'mod/bot/gagent'; end
begin require 'aliceagent'; rescue LoadError; require 'mod/bot/aliceagent'; end
begin require 'mathetesagent'; rescue LoadError; require 'mod/bot/mathetesagent'; end
begin require 'wnagent'; rescue LoadError; require 'mod/bot/wnagent'; end
begin require 'linkagent'; rescue LoadError; require 'mod/bot/linkagent'; end
begin require 'megahalagent'; rescue LoadError; require 'mod/bot/megahalagent'; end
begin require 'montyagent'; rescue LoadError; require 'mod/bot/montyagent'; end

class Controller

  attr_accessor :MY_NAME, :DEBUG, :DELAY, :SLEEP

  def initialize
    @gagent = GAgent.new(self) # port 9011
    @alice = ALICEAgent.new(self) # port 9020
    @mathetes = MathetesAgent.new(self) # port 9030
    @wnbot = WNAgent.new(self) # port 9040
    @link = LinkAgent.new(self) # port 9050
    @megahal = MegaHALAgent.new(self) # port 9060
    @monty = MontyAgent.new(self) # port 9070
    @agent_array = [ @gagent, @alice, @mathetes, @wnbot, @link, 
                     @megahal, @monty ]
    @response_array = Array.new
    @THRESHOLD = 1
    @SLEEP = 2
    @DELAY = 5
    @MY_NAME = "(?i)(controller|controlbot|controlagent)"
    @DEBUG = 1
  end

  def getResponse(input = "")
    @response_array = []
    submitInput(input)
    total_seconds = 0
    some_response = ["Default response from controller."]
    max_so_far = 0 
    # Keep a count to ensure that the loop executes at least once before 
    # the top-scoring response is determined:
    count = 0 
    # Every few seconds, loop through @response_array looking for 
    # high-scoring responses:
    loop {     
      count += 1
      #puts "#{@SLEEP}, #{@DELAY}"
      sleep(@SLEEP.to_f) 
      #puts @response_array.inspect
      @response_array.each { |response|
        #puts "response==#{response}"
        score = response[1] rescue score = 1
        if input =~ /#{@MY_NAME}[:,!]* (.*)/ then score += 2 end
        response[1] = score
        if max_so_far > @THRESHOLD and count > 1
          return some_response
        end
        if score > max_so_far
          some_response = response
          max_so_far = score
          next
        end
      }
      total_seconds += @SLEEP.to_f
      if total_seconds > @DELAY.to_f
        return some_response
      end
    }
  end

  # Send input asynchronously to agents.
  def submitInput(input)
    if input =~ /#{@MY_NAME}[:,!]* (.*)/ then input = $1 end
    @agent_array.each { |agent|
      Thread.new(agent) { |myagent|
        myagent.send(input)
      }
      # if input =~ an agent name, send it to that agent only
    #  begin
    #    if input =~ /#{agent.MY_NAME}[:,!]* / then break end
    #  rescue
    #    puts "error!"
    #  end
    }
  end

  # Callback for agents.
  def addResponse(response)
    #puts "addResponse called with: #{response}"
    @response_array.push(response)
  end

end

if $0 == __FILE__
  c = Controller.new
  print "\n> "; $stdout.flush
  quitwords = [":q", "quit", "exit", "bye"]
  while (line = gets) !~ /^#{quitwords.join('|')}$/ and line !~ /^$/
    response = c.getResponse(line.to_s).to_s + "\n\n"
    print "\n#{response}"
    print "> "; $stdout.flush
  end
  puts "Bye"
end
